package main;

import static opengl.GL.GL_COLOR_BUFFER_BIT;
import static opengl.GL.GL_DEPTH_BUFFER_BIT;
import static opengl.GL.GL_STENCIL_BUFFER_BIT;
import static opengl.GL.GL_TRIANGLES;
import static opengl.GL.glBindVertexArray;
import static opengl.GL.glClear;
import static opengl.GL.glClearColor;
import static opengl.GL.glDrawArrays;
import static opengl.GL.glGetUniformLocation;
import static opengl.GL.glPointSize;
import static opengl.GL.glUniformMatrix4;
import static opengl.GL.glUseProgram;
import static opengl.GL.init;

import java.util.logging.Level;
import java.util.logging.Logger;

import org.lwjgl.LWJGLException;
import org.lwjgl.opengl.Display;
import org.lwjgl.util.vector.Matrix4f;
import org.lwjgl.util.vector.Vector3f;

import util.Util;

/**
 * Aufbau interaktiver 3D-Engines
 * Uebungsblatt 1 Ausgabe 09.04.2013, Abgabe 19.04.2013
 * Aufgabe 2. OpenGL-Programm
 * 
 * @author Nico Marniok, Sascha Kolodzey, Erik Wittkorn
 */
public class Main {
    private static int programID;       // shader program id
    private static int triangleVAID;    // vertexarrayobject id of the pyramid
    private static int modelLocation;   // uniform location of "model"
    
    private static final Matrix4f triforceModel[] = new Matrix4f[] {
        new Matrix4f(), new Matrix4f(), new Matrix4f()
    };
    
    private static final Matrix4f view = new Matrix4f();
    private static final Matrix4f projection = new Matrix4f();
   
    private static boolean bContinue = true;

    /**
     * OpenGL wird initialisiert, das ShaderProgram wird geladen, die DreiecksGeometrie erstellt,
     * die Triforce-Rotationsmatrizen werden initialisiert und die Render-Funktion ausgefuehrt
     */
    public static void main(String[] argv) {
        try {
            init();
            programID = Util.createShaderProgram("shader/Main_VS.glsl", "shader/VertexColor_FS.glsl");
            modelLocation = glGetUniformLocation(programID, "model");
            triangleVAID = Util.createTriangle();
            glPointSize(16.0f);
            initTriforce();
            render();
            Display.destroy();
        } catch (LWJGLException ex) {
            Logger.getLogger(Main.class.getName()).log(Level.SEVERE, null, ex);
        }
    }
    
    /**
     * Die Dreiecksgeometrie wird durch Iteration durch die Transformationsmatrizen
     * dreimal mit verschiedener Transformation gezeichnet
     */
    public static void render() throws LWJGLException {
        glClearColor(0.0f, 0.0f, 0.0f, 1.0f); // black
        long last = System.currentTimeMillis();
        long now;
        while(bContinue && !Display.isCloseRequested()) {            
            glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);
            glUseProgram(programID);
            
            for(Matrix4f triforceTriangle : triforceModel) {
            	triforceTriangle.store(Util.MAT_BUFFER);
                Util.MAT_BUFFER.position(0);
                glUniformMatrix4(modelLocation, false, Util.MAT_BUFFER);
                glBindVertexArray(triangleVAID);
                glDrawArrays(GL_TRIANGLES, 0, 3);
            }
            
            Display.update();
            Display.sync(60);
            
            now = System.currentTimeMillis();
            updateUniforms();
            last = now;
        }
    }
    
    /**
     * Die Transformationsmatrizen fuer das Triforce werden initialisiert
     */
    public static void initTriforce() {
    	Matrix4f scale = Util.scale(new Vector3f(1/2f, 1/2f,0f), null);
    	
    	triforceModel[0] = Util.mul(null, Util.translationX(-0.5f, null), Util.translationY((-1/3f) - (1/6f), null), scale);
    	triforceModel[1] = Util.mul(null, Util.translationX(0.5f, null), Util.translationY((-1/3f) - (1/6f), null), scale);
    	triforceModel[2] = Util.mul(null, Util.translationX(0.0f, null), Util.translationY((2/3f) - (1/6f), null), scale);
    }
    
    /**
     * Die Uniform-Variablen und die view-Projection-Matrix im speziellen wird im ShaderProgram geupdated
     */
    public static void updateUniforms() {     
        Matrix4f viewProj = Matrix4f.mul(Main.projection, Main.view, null);
        viewProj.store(Util.MAT_BUFFER);
        Util.MAT_BUFFER.position(0);
        int viewProjLoc = glGetUniformLocation(programID, "viewProj");
        glUniformMatrix4(viewProjLoc, false, Util.MAT_BUFFER);
    }
}
