package model.components;

import model.Actor;

/**
 * Prototypische abstrakte Klasse, von der alle Komponenten
 * eines Actors abzuleiten sind
 * 
 * @author Erik Wittkorn
 */
public abstract class ActorComponent {
	
	/**
	 * Jede Komponente hat eine Referenz auf ihren Besitzer
	 */
	private Actor owner;
	/**
	 * Jede Komponente hat einen eindeutigen Namen
	 */
	private String name;
	
	/**
	 * Der Name der Komponente soll im Konstruktor der abgeleiteten
	 * Komponente gesetzt werden
	 * @param name 
	 */
	final void setName(String name) {
		this.name = name;
	}
	
	public final String getName() {
		return this.name;
	}
	
    /**
     * Setzt den besitzenden Actor.
     * @param owner Der besitzende Actor
     */
    public final void setOwner(Actor owner) {
        this.owner = owner;
    }
    
    /**
     * Getter fuer den besitzenden Actor.
     * @return Der besitzende Actor
     */
    public final Actor getOwner() {
        return this.owner;
    }
    
    /**
     * Muss von jeder Komponente ueberschrieben werden und gibt
     * alle verfuegbaren Informationen ueber diese Komponente als
     * String zurueck
     */
    public abstract String toString();

}
