package model.components;

import java.util.ArrayList;
import java.util.List;

import model.Actor;
import model.Box;

/**
 * Prototypische Komponente um die Physikalischen Eigenschaften eines
 * Actors zu beschreiben
 * 
 * Ist von der allgemeinen Actor-Komponente abgeleitet
 * 
 * @author Erik Wittkorn
 */
public class PhysicsComponent extends ActorComponent {

	/**
	 * Der physikalische Koerper wird in einer Liste von
	 * Box-Objekten festgehalten
	 */
	private final ArrayList<Box> boxes = new ArrayList<Box>();
	
	/**
	 * Im Konstruktor wird der eindeutige Name dieser Komponente eingetragen
	 * und die Owner-Referenz auf null gesetzt
	 */
	public PhysicsComponent () {
		this.setName("PhysicsComponent");
		this.setOwner(null);
	}
	
	public void addBox(Box box) {
		this.boxes.add(box);
	}
	
	public ArrayList<Box> getBoxes() {
		return this.boxes;
	}

	/**
	 * Siehe Oberklasse
	 */
	@Override
	public String toString() {
		//TODO Aussagekraeftige Information ueber ein Objekt dieser Klasse ausgeben
		return "";
	}
}
