package writer;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.util.ArrayList;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import model.Actor;
import model.BoundingSphere;
import model.Box;
import model.components.PhysicsComponent;
import model.components.RenderComponent;
import model.components.TransformComponent;

import org.lwjgl.util.vector.Vector3f;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * Hilfsklasse, um aus einer Actor-Liste eine entsprechende XML-Datei
 * zu generieren
 * 
 * @author Erik Wittkorn
 */
public class SceneXMLWriter {

	/**
	 * Durch Hilfsfunktionen wird hier aus einer Actor-Liste eine
	 * entsprechende XML-Datei generiert und in eine gewuenschte
	 * Datei geschrieben
	 * @param actorList Liste mit Actors
	 * @param fileName gequenschter Dateiname fuer die Ausgabe-XML
	 */
	public static void writeSceneXML(ArrayList<Actor> actorList, String fileName) {
		Document sceneDocument = createSceneDocument(actorList);
		try {
			FileOutputStream fileStream = new FileOutputStream(new File(fileName));
			printDocument(sceneDocument, fileStream);
			fileStream.close();
		} catch (IOException | TransformerException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * In dieser Hilfsfunktion wird aus einer Actor-Liste ein
	 * Document-Object generiert, das den Inhalt der Liste repraesentiert
	 * @param actorList Liste mit Actors
	 * @return Document-Object der entsprechenden XML
	 */
	private static Document createSceneDocument(ArrayList<Actor> actorList) {
		Document sceneDocument = null;
		
		//TODO: Aus der Actor-Liste ein DOM-Document-Object erstellen
		
		return sceneDocument;
	}
	
	/**
	 * Hilfsfunktion, um ein beliebiges Document-Object als XML formatiert in einen
	 * beliebigen OutputStream zu schreiben
	 * @param doc Document-Object der XML
	 * @param out OutputStream fuer das XML-Ergebnis
	 * @throws IOException
	 * @throws TransformerException
	 */
	private static void printDocument(Document doc, OutputStream out) throws IOException, TransformerException {
        TransformerFactory tf = TransformerFactory.newInstance();
        Transformer transformer = tf.newTransformer();
        transformer.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION, "no");
        transformer.setOutputProperty(OutputKeys.METHOD, "xml");
        transformer.setOutputProperty(OutputKeys.INDENT, "yes");
        transformer.setOutputProperty(OutputKeys.ENCODING, "UTF-8");
        transformer.setOutputProperty("{http://xml.apache.org/xslt}indent-amount", "4");

        transformer.transform(new DOMSource(doc), 
             new StreamResult(new OutputStreamWriter(out, "UTF-8")));
    }
}
