package parser;

import java.io.FileReader;
import java.io.IOException;

import model.RectangleShape;

import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLReaderFactory;

import parser.sax.SAXRectangleShapeParserHandler;

/**
 * Prototypische Klasse, in der eine XML-Datei mit einem <CubeShape>-Element nach
 * der SAX-API geparst wird
 */
public class SAXRectangleShapeParser {

	public static RectangleShape parse (String filename) {
		RectangleShape cubeShape = new RectangleShape();
		try {
			// Die XML-Datei wird eingelesen
			FileReader reader = new FileReader(filename);
			InputSource inputSource = new InputSource(reader);
			
			// Der XML-Reader wird erstellt
			XMLReader xmlReader = XMLReaderFactory.createXMLReader();
			// Dem Handler-Objekt wird ein Standard CubeShape-Objekte fuer das Ergebnis aus der XML-Datei uebergeben
			SAXRectangleShapeParserHandler cubeShapeHandler = new SAXRectangleShapeParserHandler(cubeShape);
			xmlReader.setContentHandler(cubeShapeHandler);
			
			//Die XML-Datei wird geparst
			xmlReader.parse(inputSource);
			
		} catch (SAXException | IOException e) {
			e.printStackTrace();
		}
		
		return cubeShape;
	}
}
